This guide will cover how to make two specific fs patches, with System Firmware Version 13.0.0 targeted.

**note:** the patch only has to be done on either fat32 or exfat, you just need the sha256 of the other fs.kip1 

As making FS patches require dumping firmware files, extracting ncas, dumping romfs, extracking package2, sha256'ing uncompressed binaries for both fat32 and exfat, then obtaining a decompressed version of one of them, it is a lot of effort involved when not just automating it. An example of python script to automate the organizing and extraction of things: [firmware_organizer.py](../scripts/firmware_organizer.py) Usage: create a folder called "firmware", put firmware .ncas inside of the "firmware" folder then use "python firmware_organizer.py" (this should also be covered in Part 2 of the tutorial)

1. Obtain firmware files dumped from either your own console, or from https://darthsternie.net/switch-firmwares/ (as described in Part 2)

2. Extract the required binaries by running the firmware_organizer.py with python. (place .nca files in a sub-folder called "firmware"). The process the python script does looks like this, if you have an interest in how it is done:
   - for the fat32 the titleid is 0100000000000819
   - for the exfat the titleid is 010000000000081B
   - hactool firmware\0100000000000819\c0cb231da1d3706d346d189779ba82f3.nca --romfsdir fat32romfs
   - hactool firmware\010000000000081B\adc64b133167ad1c41a1a3de148cc73c.nca --romfsdir exfatromfs
   - hactool -t pk21 fat32romfs/nx/package2 --ini1dir fat32
   - hactool -t pk21 exfatromfs/nx/package2 --ini1dir exfat
   - hactool -t kip1 fat32romfs/nx/package2/fat32/fs.kip1 --uncompressed=uncompressed_fat32.kip1
   - hactool -t kip1 exfatromfs/nx/package2/fat32/fs.kip1 --uncompressed=uncompressed_exfat.kip1

* Optional steps to get the sha256, which are printed automatically in step 2: 
  - Get filehashes for both compressed fat32 and exfat binaries, you can get them by doing the following:
  - certUtil -hashfile fat32/fs.kip1 SHA256 (for windows cmd prompt)
  - Get-Filehash fat32/Fs.kip1 (for windows powershell)
  - sha256sum fat32/Fs.kip1 for linux terminal
  - Write this hash down. (7D200547178A836A24993B002AFD83D0B662AA7177E935C04C36EAC21BAC24AA fat32 13.0.0)
  - repeat same process for the compressed exfat Fs.kip1
  - Write this hash down. (51EBFA9CCF66C09EC796C069C6C6D03C01D504CC0B17F4FAA919446E7E9DB107 exfat 13.0.0)

3. Import uncompressed_fat32.kip1 into ghidra, analyze it, wait until process complete. (you don't have to do this for the exfat binary)
   - press ok if there are any errors

4. Next, get the first offset that you are going to be applying one of the patches to.
   - for the first patch press S, search for hex string: ".. .. .. 12 .. .. .. 71 .. .. .. 54 .. .. .. 12 .. .. .. 71 .. .. .. 54 .. .. .. 36 .. .. .. f9" (the offset you're patching is the the BL instruction right after the search result) This search string is for the instruction pattern at the function that is being patched, it permits for installing .xci sourced ncas without converting the header.
   - patch the offset (as pictured) with a NOP instruction (1F2003D5)
![alt text](https://github.com/borntohonk/SigPatches/blob/master/img/ghidra-fs-offset-1.png?raw=true)

5. Then, get the second offset that you are going to be applying one of the patches to.
   - for the second patch press S, search for hex string: "1e 42 b9 1f c1 42 71" (the offset you're patching is the TBZ arm instruction below the search result), the offset desired is -0x5 from the result.
   - patch offset (as pictured) with a MOV, w0, WZR instruction. (E0031F2A) (image below slighly outdated)
![alt text](https://github.com/borntohonk/SigPatches/blob/master/img/ghidra-fs-offset-2.png?raw=true)

6. Save the end result as string after merging the output offset, sizes and patches to be applied, remember to correct the offsets for .ips patches by adding 0x100, for hekate based patches one does not need to do this offset correction.
   - (50 41 54 43 48 07 AF C8 00 04 E0 03 1F 2A 0E F4 20 00 04 1F 20 03 D5 45 4F 46) as 7D200547178A836A24993B002AFD83D0B662AA7177E935C04C36EAC21BAC24AA.ips and 51EBFA9CCF66C09EC796C069C6C6D03C01D504CC0B17F4FAA919446E7E9DB107.ips (for 13.0.0, images are from 12.0.3/12.1.0) (You can look at the table below to get a more visual concept of how this patch is structured)

7. It should be noted that the logic for fs patches changed as of firmware version 14.0.0 and that you now need to make different patches for both fat32 and exfat.

| PATCH magic | Offset 1 | Size 1 | Patch 1 | Offset 2 | Size 2 | Patch 2 | EOF magic |
| :---: | :---: | :---: | :---: | :---: | :---: | :---: | :---: |
| 50 41 54 43 48 | 07 AF C8 | 00 04 | E0 03 1F 2A | 0E F4 20 | 00 04 | 1F 20 03 D5| 45 4F 46 |

You now have succesfully patched out two different checks that prevent installing homebrew/xci and it can be used with Atmosphere by putting it into /atmosphere/kip_patches/fs_patches/32153D9896AE0C7C723F04EC1CE20999CAF7ED69.ips and /atmosphere/kip_patches/fs_patches/51EBFA9CCF66C09EC796C069C6C6D03C01D504CC0B17F4FAA919446E7E9DB107.ips

* [Python script to generate FS patches, usage: place uncompressed_exfat.kip1, compressed_exfat.kip1 and compressed_fat32.kip1 in the same folder then use "python3 fs_patch.py"](../scripts/fs_patch.py)
